@@ -48,7 +48,7 @@ check_reports_valid <- function(data,
     assert_numeric(data$confirm, lower = 0)
   }
   assert_logical(data$accumulate, null.ok = TRUE)
-  return(invisible(data))
+  invisible(data)
 }
 
 #' Validate probability distribution for passing to stan
@@ -90,7 +90,7 @@ check_stan_delay <- function(dist) {
       vapply(params, function(x) {
         is.numeric(x) ||
           (is(x, "dist_spec") && get_distribution(x) == "normal" &&
-            is.infinite(max(x)))
+             is.infinite(max(x)))
       }, logical(1))
     }
   }))
@@ -109,7 +109,7 @@ check_stan_delay <- function(dist) {
   }
   assert_numeric(attr(dist, "cdf_cutoff"), lower = 0, upper = 1)
   # Check that `dist` has a finite maximum
-  if (any(is.infinite(max(dist))) && !(attr(dist, "cdf_cutoff") > 0)) {
+  if (any(is.infinite(max(dist))) && attr(dist, "cdf_cutoff") == 0) {
     cli_abort(
       c(
         "i" = "All distributions passed to the model need to have a
@@ -180,3 +180,75 @@ check_sparse_pmf_tail <- function(pmf, span = 5, tol = 1e-6) {
     )
   }
 }
+
+#' Check and warn if truncation distribution is longer than observed time
+#'
+#' @description Checks if the truncation distribution PMF is longer than the
+#' observed time period (excluding seeding time and forecast horizon). The
+#' truncation is applied to the observed time period in the Stan model, so
+#' having a truncation distribution longer than this period means the tail of
+#' the distribution will be used.
+#'
+#' @param stan_args List of stan arguments including the data element with
+#'   delay information from [create_stan_delays()]
+#' @param time_points Integer length of the observed time period
+#'   (t - seeding_time - horizon)
+#' @importFrom cli cli_warn col_blue
+#'
+#' @return Called for its side effects
+#' @keywords internal
+check_truncation_length <- function(stan_args, time_points) {
+  # Check if truncation exists
+  if (is.null(stan_args$data$delay_id_truncation) ||
+        stan_args$data$delay_id_truncation == 0) {
+    return(invisible())
+  }
+
+  # Check if there are any non-parametric delays
+  if (is.null(stan_args$data$delay_n_np) || stan_args$data$delay_n_np == 0) {
+    return(invisible())
+  }
+
+  # Map truncation to its position in the flat delays array
+  # delay_types_groups gives start and end indices for each delay type
+  trunc_start <- stan_args$data$delay_types_groups[
+    stan_args$data$delay_id_truncation
+  ]
+  trunc_end <- stan_args$data$delay_types_groups[
+    stan_args$data$delay_id_truncation + 1
+  ] - 1
+
+  # Get which truncation delays are non-parametric
+  trunc_range <- trunc_start:trunc_end
+  is_np <- stan_args$data$delay_types_p[trunc_range] == 0
+
+  # Return early if all truncation delays are parametric
+  if (!any(is_np)) {
+    return(invisible())
+  }
+
+  # Get the IDs of non-parametric truncation delays within the np array
+  np_trunc_ids <- stan_args$data$delay_types_id[trunc_range[is_np]]
+
+  # Calculate individual PMF lengths from the indices
+  np_pmf_lengths <- diff(stan_args$data$delay_np_pmf_groups)
+
+  # Extract truncation PMF length(s)
+  trunc_pmf_lengths <- np_pmf_lengths[np_trunc_ids]
+
+  # Check if any truncation PMF exceeds time_points
+  if (any(trunc_pmf_lengths > time_points)) {
+    cli::cli_warn(
+      c(
+        "!" = "The truncation distribution is longer than the observed time
+        period.",
+        "i" = "The truncation distribution has length
+        {col_blue({max(trunc_pmf_lengths)})} but the observed time period is
+        {col_blue(time_points)} days. The tail of the truncation distribution
+        will be used."
+      ),
+      .frequency = "once",
+      .frequency_id = "truncation_longer_than_data"
+    )
+  }
+}
