@@ -27,18 +27,14 @@
 #' @keywords internal
 fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
                                 id = "stan") {
+  method <- args$method
   args$method <- NULL
   args$max_execution_time <- NULL
   args$future <- NULL
 
+  log_msg <- create_sampling_log_message(args, method)
   futile.logger::flog.debug(
-    paste0(
-      "%s: Running in exact mode for ",
-      ceiling(args$iter - args$warmup) * args$chains,
-      " samples (across ", args$chains,
-      " chains each with a warm up of ", args$warmup, " iterations each) and ",
-      args$data$t, " time steps of which ", args$data$horizon, " are a forecast"
-    ),
+    log_msg,
     id,
     name = "EpiNow2.epinow.estimate_infections.fit"
   )
@@ -80,7 +76,7 @@ fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
           futile.logger::flog.error(error_text,
             name = "EpiNow2.epinow.estimate_infections.fit"
           )
-          return(NULL)
+          NULL
         }
       )
     } else {
@@ -91,10 +87,10 @@ fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
     }
 
     if ((inherits(fit, "stanfit") && fit@mode != 2L) ||
-      inherits(fit, "CmdStanMCMC")) {
-      return(fit)
+          inherits(fit, "CmdStanMCMC")) {
+      fit
     } else {
-      return(NULL)
+      NULL
     }
   }
 
@@ -102,14 +98,14 @@ fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
     chains <- args$chains
     args$chains <- 1
     args$cores <- 1
-    fits <- lapply_func(1:chains,
+    fits <- lapply_func(seq_len(chains),
       fit_chain,
       stan_args = args,
       max_time = max_execution_time,
       catch = TRUE
     )
     if (stuck_chains > 0) {
-      fits[1:stuck_chains] <- NULL
+      fits[seq_len(stuck_chains)] <- NULL
     }
     fit <- purrr::compact(fits)
     if (length(fit) == 0) {
@@ -148,7 +144,7 @@ fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
       rlang::abort("model fitting was timed out or failed")
     }
   }
-  return(fit)
+  fit
 }
 
 #' Fit a Stan Model using an approximate method
@@ -168,14 +164,9 @@ fit_model_with_nuts <- function(args, future = FALSE, max_execution_time = Inf,
 fit_model_approximate <- function(args, future = FALSE, id = "stan") {
   method <- args$method
   args$method <- NULL
+  log_msg <- create_sampling_log_message(args, method)
   futile.logger::flog.debug(
-    paste0(
-      "%s: Running in approximate mode for ", args$iter,
-      " iterations (with ", args$trials, " attempts). Extracting ",
-      args$output_samples, " approximate posterior samples for ",
-      args$data$t, " time steps of which ",
-      args$data$horizon, " are a forecast"
-    ),
+    log_msg,
     id,
     name = "EpiNow2.epinow.estimate_infections.fit"
   )
@@ -213,11 +204,10 @@ fit_model_approximate <- function(args, future = FALSE, id = "stan") {
     fit <- do.call(sample_func, stan_args)
 
     if (length(names(fit)) == 0) {
-      return(NULL)
+      NULL
     } else {
-      return(fit)
+      fit
     }
-    return(fit)
   }
   safe_fit <- purrr::safely(fit_approximate) # nolint
   fit <- NULL
@@ -246,5 +236,88 @@ fit_model_approximate <- function(args, future = FALSE, id = "stan") {
     )
     rlang::abort(paste("Approximate inference failed due to:", error))
   }
-  return(fit)
+  fit
+}
+
+#' Create sampling log message
+#'
+#' @description Internal function that creates a formatted log message
+#' describing the sampling parameters. The message format varies by method,
+#' with different information shown for exact sampling vs approximate methods
+#' (VB, Laplace, Pathfinder). Optionally includes time steps and forecast
+#' horizon if present.
+#'
+#' @param args List of stan arguments containing:
+#'   - object: Stan model object (CmdStanModel or stanmodel)
+#'   - For sampling method: iter_sampling, iter_warmup (cmdstanr) or iter,
+#'     warmup (rstan), chains
+#'   - For vb method: iter, trials, output_samples
+#'   - For laplace method: trials
+#'   - For pathfinder method: trials, draws
+#'   - data: List potentially containing t (time steps) and horizon (forecast)
+#' @param method Character string indicating the sampling method. One of
+#'   "sampling" (exact MCMC), "vb" (variational Bayes), "laplace" (Laplace
+#'   approximation), or "pathfinder" (pathfinder algorithm).
+#'
+#' @return A character string containing the formatted log message with a %s
+#'   placeholder for the id parameter (to be filled by sprintf or flog.debug)
+#'
+#' @keywords internal
+create_sampling_log_message <- function(args, method) {
+  # Build base message depending on method
+  log_msg <- switch(method,
+    "sampling" = {
+      # Exact mode - calculate parameters based on backend
+      if (inherits(args$object, "CmdStanModel")) {
+        total_samples <- args$iter_sampling * args$chains
+        warmup_iterations <- args$iter_warmup
+      } else {
+        total_samples <- (args$iter - args$warmup) * args$chains
+        warmup_iterations <- args$warmup
+      }
+
+      paste0(
+        "%s: Running in exact mode for ", total_samples,
+        " samples (across ", args$chains,
+        " chains each with a warm up of ", warmup_iterations,
+        " iterations each)"
+      )
+    },
+    "vb" = {
+      # VB approximate mode
+      paste0(
+        "%s: Running in approximate mode for ", args$iter,
+        " iterations (with ", args$trials, " attempts). Extracting ",
+        args$output_samples, " approximate posterior samples"
+      )
+    },
+    "laplace" = {
+      # Laplace approximate mode
+      paste0(
+        "%s: Running in approximate mode using Laplace approximation (with ",
+        args$trials, " attempts)"
+      )
+    },
+    "pathfinder" = {
+      # Pathfinder approximate mode
+      paste0(
+        "%s: Running in approximate mode using pathfinder (with ",
+        args$trials, " attempts). Extracting ",
+        args$draws, " approximate posterior samples"
+      )
+    }
+  )
+
+  # Add time steps if present (for all methods)
+  if (!is.null(args$data$t)) {
+    log_msg <- paste0(log_msg, " for ", args$data$t, " time steps")
+    # Add forecast horizon if present
+    if (!is.null(args$data$horizon)) {
+      log_msg <- paste0(
+        log_msg, " of which ", args$data$horizon, " are a forecast"
+      )
+    }
+  }
+
+  log_msg
 }
