@@ -1,6 +1,6 @@
 # Vignettes that have long run times
 
-library("knitr")
+library(knitr)
 orig_files <- list.files(pattern = "Rmd.orig$")
 for (f in orig_files) {
   knit(f, sub(".orig$", "", f))
